/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
.md-inline-form md-checkbox {
  margin: 19px 0 18px; }

md-checkbox {
  box-sizing: border-box;
  display: inline-block;
  margin-bottom: 16px;
  white-space: nowrap;
  cursor: pointer;
  outline: none;
  -webkit-user-select: none;
     -moz-user-select: none;
      -ms-user-select: none;
          user-select: none;
  position: relative;
  min-width: 20px;
  min-height: 20px;
  margin-left: inherit;
  margin-right: 16px; }
  html[dir=rtl] md-checkbox {
    margin-left: 16px;
    unicode-bidi: embed; }
  body[dir=rtl] md-checkbox {
    margin-left: 16px;
    unicode-bidi: embed; }
  md-checkbox bdo[dir=rtl] {
    direction: rtl;
    unicode-bidi: bidi-override; }
  md-checkbox bdo[dir=ltr] {
    direction: ltr;
    unicode-bidi: bidi-override; }
  html[dir=rtl] md-checkbox {
    margin-right: inherit;
    unicode-bidi: embed; }
  body[dir=rtl] md-checkbox {
    margin-right: inherit;
    unicode-bidi: embed; }
  md-checkbox bdo[dir=rtl] {
    direction: rtl;
    unicode-bidi: bidi-override; }
  md-checkbox bdo[dir=ltr] {
    direction: ltr;
    unicode-bidi: bidi-override; }
  md-checkbox:last-of-type {
    margin-left: inherit;
    margin-right: 0; }
    html[dir=rtl] md-checkbox:last-of-type {
      margin-left: 0;
      unicode-bidi: embed; }
    body[dir=rtl] md-checkbox:last-of-type {
      margin-left: 0;
      unicode-bidi: embed; }
    md-checkbox:last-of-type bdo[dir=rtl] {
      direction: rtl;
      unicode-bidi: bidi-override; }
    md-checkbox:last-of-type bdo[dir=ltr] {
      direction: ltr;
      unicode-bidi: bidi-override; }
    html[dir=rtl] md-checkbox:last-of-type {
      margin-right: inherit;
      unicode-bidi: embed; }
    body[dir=rtl] md-checkbox:last-of-type {
      margin-right: inherit;
      unicode-bidi: embed; }
    md-checkbox:last-of-type bdo[dir=rtl] {
      direction: rtl;
      unicode-bidi: bidi-override; }
    md-checkbox:last-of-type bdo[dir=ltr] {
      direction: ltr;
      unicode-bidi: bidi-override; }
  md-checkbox.md-focused:not([disabled]) .md-container:before {
    left: -8px;
    top: -8px;
    right: -8px;
    bottom: -8px; }
  md-checkbox.md-focused:not([disabled]):not(.md-checked) .md-container:before {
    background-color: rgba(0, 0, 0, 0.12); }
  md-checkbox .md-container {
    position: absolute;
    top: 50%;
    -webkit-transform: translateY(-50%);
            transform: translateY(-50%);
    box-sizing: border-box;
    display: inline-block;
    width: 20px;
    height: 20px;
    left: 0;
    right: auto; }
    html[dir=rtl] md-checkbox .md-container {
      left: auto;
      unicode-bidi: embed; }
    body[dir=rtl] md-checkbox .md-container {
      left: auto;
      unicode-bidi: embed; }
    md-checkbox .md-container bdo[dir=rtl] {
      direction: rtl;
      unicode-bidi: bidi-override; }
    md-checkbox .md-container bdo[dir=ltr] {
      direction: ltr;
      unicode-bidi: bidi-override; }
    html[dir=rtl] md-checkbox .md-container {
      right: 0;
      unicode-bidi: embed; }
    body[dir=rtl] md-checkbox .md-container {
      right: 0;
      unicode-bidi: embed; }
    md-checkbox .md-container bdo[dir=rtl] {
      direction: rtl;
      unicode-bidi: bidi-override; }
    md-checkbox .md-container bdo[dir=ltr] {
      direction: ltr;
      unicode-bidi: bidi-override; }
    md-checkbox .md-container:before {
      box-sizing: border-box;
      background-color: transparent;
      border-radius: 50%;
      content: '';
      position: absolute;
      display: block;
      height: auto;
      left: 0;
      top: 0;
      right: 0;
      bottom: 0;
      transition: all 0.5s;
      width: auto; }
    md-checkbox .md-container:after {
      box-sizing: border-box;
      content: '';
      position: absolute;
      top: -10px;
      right: -10px;
      bottom: -10px;
      left: -10px; }
    md-checkbox .md-container .md-ripple-container {
      position: absolute;
      display: block;
      width: auto;
      height: auto;
      left: -15px;
      top: -15px;
      right: -15px;
      bottom: -15px; }
  md-checkbox.md-align-top-left > div.md-container {
    top: 12px; }
  md-checkbox .md-icon {
    box-sizing: border-box;
    transition: 240ms;
    position: absolute;
    top: 0;
    left: 0;
    width: 20px;
    height: 20px;
    border-width: 2px;
    border-style: solid;
    border-radius: 2px; }
  md-checkbox.md-checked .md-icon {
    border: none; }
  md-checkbox[disabled] {
    cursor: default; }
  md-checkbox.md-checked .md-icon:after {
    box-sizing: border-box;
    -webkit-transform: rotate(45deg);
            transform: rotate(45deg);
    position: absolute;
    left: 6.66667px;
    top: 2.22222px;
    display: table;
    width: 6.66667px;
    height: 13.33333px;
    border-width: 2px;
    border-style: solid;
    border-top: 0;
    border-left: 0;
    content: ''; }
  md-checkbox .md-label {
    box-sizing: border-box;
    position: relative;
    display: inline-block;
    vertical-align: middle;
    white-space: normal;
    -webkit-user-select: text;
       -moz-user-select: text;
        -ms-user-select: text;
            user-select: text;
    margin-left: 30px;
    margin-right: 0; }
    html[dir=rtl] md-checkbox .md-label {
      margin-left: 0;
      unicode-bidi: embed; }
    body[dir=rtl] md-checkbox .md-label {
      margin-left: 0;
      unicode-bidi: embed; }
    md-checkbox .md-label bdo[dir=rtl] {
      direction: rtl;
      unicode-bidi: bidi-override; }
    md-checkbox .md-label bdo[dir=ltr] {
      direction: ltr;
      unicode-bidi: bidi-override; }
    html[dir=rtl] md-checkbox .md-label {
      margin-right: 30px;
      unicode-bidi: embed; }
    body[dir=rtl] md-checkbox .md-label {
      margin-right: 30px;
      unicode-bidi: embed; }
    md-checkbox .md-label bdo[dir=rtl] {
      direction: rtl;
      unicode-bidi: bidi-override; }
    md-checkbox .md-label bdo[dir=ltr] {
      direction: ltr;
      unicode-bidi: bidi-override; }
