/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-switch.md-THEME_NAME-theme .md-ink-ripple {
  color: '{{background-500}}'; }

md-switch.md-THEME_NAME-theme .md-thumb {
  background-color: '{{background-50}}'; }

md-switch.md-THEME_NAME-theme .md-bar {
  background-color: '{{background-500}}'; }

md-switch.md-THEME_NAME-theme.md-checked .md-ink-ripple {
  color: '{{accent-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked .md-thumb {
  background-color: '{{accent-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked .md-bar {
  background-color: '{{accent-color-0.5}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-focused .md-thumb:before {
  background-color: '{{accent-color-0.26}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-ink-ripple {
  color: '{{primary-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-thumb {
  background-color: '{{primary-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-primary .md-bar {
  background-color: '{{primary-color-0.5}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-primary.md-focused .md-thumb:before {
  background-color: '{{primary-color-0.26}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-ink-ripple {
  color: '{{warn-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-thumb {
  background-color: '{{warn-color}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-warn .md-bar {
  background-color: '{{warn-color-0.5}}'; }

md-switch.md-THEME_NAME-theme.md-checked.md-warn.md-focused .md-thumb:before {
  background-color: '{{warn-color-0.26}}'; }

md-switch.md-THEME_NAME-theme[disabled] .md-thumb {
  background-color: '{{background-400}}'; }

md-switch.md-THEME_NAME-theme[disabled] .md-bar {
  background-color: '{{foreground-4}}'; }
