/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-tooltip.md-THEME_NAME-theme {
  color: '{{background-A100}}'; }
  md-tooltip.md-THEME_NAME-theme .md-content {
    background-color: '{{foreground-2}}'; }
