/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-progress-linear.md-THEME_NAME-theme .md-container {
  background-color: '{{primary-100}}'; }

md-progress-linear.md-THEME_NAME-theme .md-bar {
  background-color: '{{primary-color}}'; }

md-progress-linear.md-THEME_NAME-theme.md-warn .md-container {
  background-color: '{{warn-100}}'; }

md-progress-linear.md-THEME_NAME-theme.md-warn .md-bar {
  background-color: '{{warn-color}}'; }

md-progress-linear.md-THEME_NAME-theme.md-accent .md-container {
  background-color: '{{accent-100}}'; }

md-progress-linear.md-THEME_NAME-theme.md-accent .md-bar {
  background-color: '{{accent-color}}'; }

md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-warn .md-bar1 {
  background-color: '{{warn-100}}'; }

md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-warn .md-dashed:before {
  background: radial-gradient("{{warn-100}}" 0%, "{{warn-100}}" 16%, transparent 42%); }

md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-accent .md-bar1 {
  background-color: '{{accent-100}}'; }

md-progress-linear.md-THEME_NAME-theme[md-mode=buffer].md-accent .md-dashed:before {
  background: radial-gradient("{{accent-100}}" 0%, "{{accent-100}}" 16%, transparent 42%); }
