<md-toolbar layout="row" layout-align="center center" layout-padding>
		<md-whiteframe  class="md-whiteframe-4dp" flex="30" layout="row" layout-align="center center">
			<div flex="70" ng-cloak>Reports for group {% verbatim %}{{absUrl.split("=")[1]}}{% endverbatim %}</div>
		</md-whiteframe>	
</md-toolbar>
	