/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-backdrop {
  background-color: '{{background-900-0.0}}'; }
  md-backdrop.md-opaque.md-THEME_NAME-theme {
    background-color: '{{background-900-1.0}}'; }
