/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-list {
  display: block;
  padding: 8px 0px 8px 0px; }
  md-list .md-subheader {
    font-size: 14px;
    font-weight: 500;
    letter-spacing: 0.010em;
    line-height: 1.2em; }

md-list-item {
  position: relative; }
  md-list-item.md-proxy-focus.md-focused .md-no-style {
    transition: background-color 0.15s linear; }
  md-list-item.md-no-proxy,
  md-list-item .md-no-style {
    position: relative;
    padding: 0px 16px;
    -webkit-flex: 1 1 auto;
        -ms-flex: 1 1 auto;
            flex: 1 1 auto; }
    md-list-item.md-no-proxy.md-button,
    md-list-item .md-no-style.md-button {
      font-size: inherit;
      height: inherit;
      text-align: left;
      text-transform: none;
      width: 100%;
      white-space: normal;
      -webkit-flex-direction: inherit;
          -ms-flex-direction: inherit;
              flex-direction: inherit;
      -webkit-align-items: inherit;
          -ms-flex-align: inherit;
              align-items: inherit;
      border-radius: 0; }
      md-list-item.md-no-proxy.md-button > .md-ripple-container,
      md-list-item .md-no-style.md-button > .md-ripple-container {
        border-radius: 0; }
    md-list-item.md-no-proxy:focus,
    md-list-item .md-no-style:focus {
      outline: none; }
  md-list-item.md-with-secondary {
    position: relative; }
  md-list-item.md-clickable:hover {
    cursor: pointer; }
  md-list-item md-divider {
    position: absolute;
    bottom: 0;
    left: 0;
    width: 100%; }
    md-list-item md-divider[md-inset] {
      left: 96px;
      width: calc(100% - 96px);
      margin: 0; }

md-list-item, md-list-item .md-list-item-inner {
  display: -webkit-flex;
  display: -ms-flexbox;
  display: flex;
  -webkit-justify-content: flex-start;
      -ms-flex-pack: start;
          justify-content: flex-start;
  -webkit-align-items: center;
      -ms-flex-align: center;
          align-items: center;
  min-height: 48px;
  height: auto; }
  md-list-item > div.md-primary > md-icon:not(.md-avatar-icon),
  md-list-item > div.md-secondary > md-icon:not(.md-avatar-icon),
  md-list-item > md-icon:first-child:not(.md-avatar-icon),
  md-list-item > md-icon.md-secondary:not(.md-avatar-icon), md-list-item .md-list-item-inner > div.md-primary > md-icon:not(.md-avatar-icon),
  md-list-item .md-list-item-inner > div.md-secondary > md-icon:not(.md-avatar-icon),
  md-list-item .md-list-item-inner > md-icon:first-child:not(.md-avatar-icon),
  md-list-item .md-list-item-inner > md-icon.md-secondary:not(.md-avatar-icon) {
    width: 24px;
    margin-top: 16px;
    margin-bottom: 12px;
    box-sizing: content-box; }
  md-list-item > div.md-primary > md-checkbox,
  md-list-item > div.md-secondary > md-checkbox,
  md-list-item > md-checkbox,
  md-list-item md-checkbox.md-secondary, md-list-item .md-list-item-inner > div.md-primary > md-checkbox,
  md-list-item .md-list-item-inner > div.md-secondary > md-checkbox,
  md-list-item .md-list-item-inner > md-checkbox,
  md-list-item .md-list-item-inner md-checkbox.md-secondary {
    -webkit-align-self: center;
        -ms-flex-item-align: center;
            align-self: center; }
    md-list-item > div.md-primary > md-checkbox .md-label,
    md-list-item > div.md-secondary > md-checkbox .md-label,
    md-list-item > md-checkbox .md-label,
    md-list-item md-checkbox.md-secondary .md-label, md-list-item .md-list-item-inner > div.md-primary > md-checkbox .md-label,
    md-list-item .md-list-item-inner > div.md-secondary > md-checkbox .md-label,
    md-list-item .md-list-item-inner > md-checkbox .md-label,
    md-list-item .md-list-item-inner md-checkbox.md-secondary .md-label {
      display: none; }
  md-list-item > md-icon:first-child:not(.md-avatar-icon), md-list-item .md-list-item-inner > md-icon:first-child:not(.md-avatar-icon) {
    margin-right: 32px; }
  md-list-item > md-checkbox, md-list-item .md-list-item-inner > md-checkbox {
    width: 24px;
    margin-left: 3px;
    margin-right: 29px;
    margin-top: 16px; }
  md-list-item .md-avatar, md-list-item .md-avatar-icon, md-list-item .md-list-item-inner .md-avatar, md-list-item .md-list-item-inner .md-avatar-icon {
    margin-top: 8px;
    margin-bottom: 8px;
    margin-right: 16px;
    border-radius: 50%;
    box-sizing: content-box; }
  md-list-item .md-avatar, md-list-item .md-list-item-inner .md-avatar {
    width: 40px;
    height: 40px; }
  md-list-item .md-avatar-icon, md-list-item .md-list-item-inner .md-avatar-icon {
    padding: 8px; }
  md-list-item md-checkbox.md-secondary,
  md-list-item md-switch.md-secondary, md-list-item .md-list-item-inner md-checkbox.md-secondary,
  md-list-item .md-list-item-inner md-switch.md-secondary {
    margin-top: 0;
    margin-bottom: 0; }
  md-list-item md-checkbox.md-secondary, md-list-item .md-list-item-inner md-checkbox.md-secondary {
    margin-right: 0; }
  md-list-item md-switch.md-secondary, md-list-item .md-list-item-inner md-switch.md-secondary {
    margin-right: -6px; }
  md-list-item button.md-button.md-secondary-container, md-list-item .md-list-item-inner button.md-button.md-secondary-container {
    background-color: transparent;
    -webkit-align-self: center;
        -ms-flex-item-align: center;
            align-self: center;
    border-radius: 50%;
    margin: 0px;
    min-width: 0px; }
    md-list-item button.md-button.md-secondary-container .md-ripple,
    md-list-item button.md-button.md-secondary-container .md-ripple-container, md-list-item .md-list-item-inner button.md-button.md-secondary-container .md-ripple,
    md-list-item .md-list-item-inner button.md-button.md-secondary-container .md-ripple-container {
      border-radius: 50%; }
    md-list-item button.md-button.md-secondary-container.md-icon-button, md-list-item .md-list-item-inner button.md-button.md-secondary-container.md-icon-button {
      margin-right: -12px; }
  md-list-item .md-secondary-container,
  md-list-item .md-secondary, md-list-item .md-list-item-inner .md-secondary-container,
  md-list-item .md-list-item-inner .md-secondary {
    position: absolute;
    top: 50%;
    right: 16px;
    margin: 0 0 0 16px;
    -webkit-transform: translate3d(0, -50%, 0);
            transform: translate3d(0, -50%, 0); }
  md-list-item > .md-button.md-secondary-container > .md-secondary, md-list-item .md-list-item-inner > .md-button.md-secondary-container > .md-secondary {
    margin-left: 0;
    position: static; }
  md-list-item > p, md-list-item > .md-list-item-inner > p, md-list-item .md-list-item-inner > p, md-list-item .md-list-item-inner > .md-list-item-inner > p {
    -webkit-flex: 1;
        -ms-flex: 1;
            flex: 1;
    margin: 0; }

md-list-item.md-2-line,
md-list-item.md-2-line > .md-no-style,
md-list-item.md-3-line,
md-list-item.md-3-line > .md-no-style {
  -webkit-align-items: flex-start;
      -ms-flex-align: start;
          align-items: flex-start;
  -webkit-justify-content: center;
      -ms-flex-pack: center;
          justify-content: center; }
  md-list-item.md-2-line .md-list-item-text,
  md-list-item.md-2-line > .md-no-style .md-list-item-text,
  md-list-item.md-3-line .md-list-item-text,
  md-list-item.md-3-line > .md-no-style .md-list-item-text {
    -webkit-flex: 1;
        -ms-flex: 1;
            flex: 1;
    margin: auto;
    text-overflow: ellipsis; }
    md-list-item.md-2-line .md-list-item-text.md-offset,
    md-list-item.md-2-line > .md-no-style .md-list-item-text.md-offset,
    md-list-item.md-3-line .md-list-item-text.md-offset,
    md-list-item.md-3-line > .md-no-style .md-list-item-text.md-offset {
      margin-left: 56px; }
    md-list-item.md-2-line .md-list-item-text h3,
    md-list-item.md-2-line > .md-no-style .md-list-item-text h3,
    md-list-item.md-3-line .md-list-item-text h3,
    md-list-item.md-3-line > .md-no-style .md-list-item-text h3 {
      font-size: 16px;
      font-weight: 400;
      letter-spacing: 0.010em;
      margin: 0 0 0px 0;
      line-height: 1.2em;
      overflow: hidden;
      white-space: nowrap;
      text-overflow: ellipsis; }
    md-list-item.md-2-line .md-list-item-text h4,
    md-list-item.md-2-line > .md-no-style .md-list-item-text h4,
    md-list-item.md-3-line .md-list-item-text h4,
    md-list-item.md-3-line > .md-no-style .md-list-item-text h4 {
      font-size: 14px;
      letter-spacing: 0.010em;
      margin: 3px 0 1px 0;
      font-weight: 400;
      line-height: 1.2em;
      overflow: hidden;
      white-space: nowrap;
      text-overflow: ellipsis; }
    md-list-item.md-2-line .md-list-item-text p,
    md-list-item.md-2-line > .md-no-style .md-list-item-text p,
    md-list-item.md-3-line .md-list-item-text p,
    md-list-item.md-3-line > .md-no-style .md-list-item-text p {
      font-size: 14px;
      font-weight: 500;
      letter-spacing: 0.010em;
      margin: 0 0 0 0;
      line-height: 1.6em; }

md-list-item.md-2-line,
md-list-item.md-2-line > .md-no-style {
  height: auto;
  min-height: 72px; }
  md-list-item.md-2-line.md-long-text,
  md-list-item.md-2-line > .md-no-style.md-long-text {
    margin: 1.6em; }
  md-list-item.md-2-line > .md-avatar, md-list-item.md-2-line .md-avatar-icon,
  md-list-item.md-2-line > .md-no-style > .md-avatar,
  md-list-item.md-2-line > .md-no-style .md-avatar-icon {
    margin-top: 12px; }
  md-list-item.md-2-line > md-icon:first-child,
  md-list-item.md-2-line > .md-no-style > md-icon:first-child {
    -webkit-align-self: flex-start;
        -ms-flex-item-align: start;
            align-self: flex-start; }
  md-list-item.md-2-line .md-list-item-text,
  md-list-item.md-2-line > .md-no-style .md-list-item-text {
    -webkit-flex: 1;
        -ms-flex: 1;
            flex: 1; }

md-list-item.md-3-line,
md-list-item.md-3-line > .md-no-style {
  height: auto;
  min-height: 88px; }
  md-list-item.md-3-line.md-long-text,
  md-list-item.md-3-line > .md-no-style.md-long-text {
    margin: 1.6em; }
  md-list-item.md-3-line > md-icon:first-child,
  md-list-item.md-3-line > .md-avatar,
  md-list-item.md-3-line > .md-no-style > md-icon:first-child,
  md-list-item.md-3-line > .md-no-style > .md-avatar {
    margin-top: 16px; }
