/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-menu-bar.md-THEME_NAME-theme > button.md-button {
  color: '{{foreground-2}}';
  border-radius: 2px; }

md-menu-bar.md-THEME_NAME-theme md-menu.md-open > button, md-menu-bar.md-THEME_NAME-theme md-menu > button:focus {
  outline: none;
  background: '{{background-200}}'; }

md-menu-bar.md-THEME_NAME-theme.md-open:not(.md-keyboard-mode) md-menu:hover > button {
  background-color: '{{ background-500-0.2}}'; }

md-menu-bar.md-THEME_NAME-theme:not(.md-keyboard-mode):not(.md-open) md-menu button:hover,
md-menu-bar.md-THEME_NAME-theme:not(.md-keyboard-mode):not(.md-open) md-menu button:focus {
  background: transparent; }

md-menu-content.md-THEME_NAME-theme .md-menu > .md-button:after {
  color: '{{foreground-2}}'; }

md-menu-content.md-THEME_NAME-theme .md-menu.md-open > .md-button {
  background-color: '{{ background-500-0.2}}'; }

md-toolbar.md-THEME_NAME-theme.md-menu-toolbar {
  background-color: '{{background-color}}';
  color: '{{foreground-1}}'; }
  md-toolbar.md-THEME_NAME-theme.md-menu-toolbar md-toolbar-filler {
    background-color: '{{primary-color}}';
    color: '{{primary-contrast}}'; }
    md-toolbar.md-THEME_NAME-theme.md-menu-toolbar md-toolbar-filler md-icon {
      color: '{{primary-contrast}}'; }
