/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-list.md-THEME_NAME-theme md-list-item.md-2-line .md-list-item-text h3, md-list.md-THEME_NAME-theme md-list-item.md-2-line .md-list-item-text h4,
md-list.md-THEME_NAME-theme md-list-item.md-3-line .md-list-item-text h3,
md-list.md-THEME_NAME-theme md-list-item.md-3-line .md-list-item-text h4 {
  color: '{{foreground-1}}'; }

md-list.md-THEME_NAME-theme md-list-item.md-2-line .md-list-item-text p,
md-list.md-THEME_NAME-theme md-list-item.md-3-line .md-list-item-text p {
  color: '{{foreground-2}}'; }

md-list.md-THEME_NAME-theme .md-proxy-focus.md-focused div.md-no-style {
  background-color: '{{background-100}}'; }

md-list.md-THEME_NAME-theme md-list-item > .md-avatar-icon {
  background-color: '{{foreground-3}}';
  color: '{{background-color}}'; }

md-list.md-THEME_NAME-theme md-list-item > md-icon {
  color: '{{foreground-2}}'; }
  md-list.md-THEME_NAME-theme md-list-item > md-icon.md-highlight {
    color: '{{primary-color}}'; }
    md-list.md-THEME_NAME-theme md-list-item > md-icon.md-highlight.md-accent {
      color: '{{accent-color}}'; }
