/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-select.md-THEME_NAME-theme[disabled] .md-select-value {
  border-bottom-color: transparent;
  background-image: linear-gradient(to right, "{{foreground-3}}" 0%, "{{foreground-3}}" 33%, transparent 0%); }

md-select.md-THEME_NAME-theme .md-select-value {
  border-bottom-color: '{{foreground-4}}'; }
  md-select.md-THEME_NAME-theme .md-select-value.md-select-placeholder {
    color: '{{foreground-3}}'; }

md-select.md-THEME_NAME-theme.ng-invalid.ng-dirty .md-select-value {
  color: '{{warn-A700}}' !important;
  border-bottom-color: '{{warn-A700}}' !important; }

md-select.md-THEME_NAME-theme:not([disabled]):focus .md-select-value {
  border-bottom-color: '{{primary-color}}';
  color: '{{ foreground-1 }}'; }
  md-select.md-THEME_NAME-theme:not([disabled]):focus .md-select-value.md-select-placeholder {
    color: '{{ foreground-1 }}'; }

md-select.md-THEME_NAME-theme:not([disabled]):focus.md-accent .md-select-value {
  border-bottom-color: '{{accent-color}}'; }

md-select.md-THEME_NAME-theme:not([disabled]):focus.md-warn .md-select-value {
  border-bottom-color: '{{warn-color}}'; }

md-select.md-THEME_NAME-theme[disabled] .md-select-value {
  color: '{{foreground-3}}'; }
  md-select.md-THEME_NAME-theme[disabled] .md-select-value.md-select-placeholder {
    color: '{{foreground-3}}'; }

md-select-menu.md-THEME_NAME-theme md-option[disabled] {
  color: '{{foreground-3}}'; }

md-select-menu.md-THEME_NAME-theme md-optgroup {
  color: '{{foreground-2}}'; }
  md-select-menu.md-THEME_NAME-theme md-optgroup md-option {
    color: '{{foreground-1}}'; }

md-select-menu.md-THEME_NAME-theme md-option[selected] {
  color: '{{primary-500}}'; }
  md-select-menu.md-THEME_NAME-theme md-option[selected]:focus {
    color: '{{primary-600}}'; }
  md-select-menu.md-THEME_NAME-theme md-option[selected].md-accent {
    color: '{{accent-500}}'; }
    md-select-menu.md-THEME_NAME-theme md-option[selected].md-accent:focus {
      color: '{{accent-600}}'; }

md-select-menu.md-THEME_NAME-theme md-option:focus:not([disabled]):not([selected]) {
  background: '{{background-200}}'; }
