/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) {
  background-color: '{{primary-color}}';
  color: '{{primary-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) md-icon {
    color: '{{primary-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar) .md-button:not(.md-raised) {
    color: '{{primary-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-accent {
    background-color: '{{accent-color}}';
    color: '{{accent-contrast}}'; }
  md-toolbar.md-THEME_NAME-theme:not(.md-menu-toolbar).md-warn {
    background-color: '{{warn-color}}';
    color: '{{warn-contrast}}'; }
