/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
md-card.md-THEME_NAME-theme {
  background-color: '{{background-color}}';
  border-radius: 2px; }
  md-card.md-THEME_NAME-theme .md-card-image {
    border-radius: 2px 2px 0 0; }
  md-card.md-THEME_NAME-theme md-card-header md-card-avatar md-icon {
    color: '{{background-color}}';
    background-color: '{{foreground-3}}'; }
  md-card.md-THEME_NAME-theme md-card-header md-card-header-text .md-subhead {
    color: '{{foreground-2}}'; }
  md-card.md-THEME_NAME-theme md-card-title md-card-title-text:not(:only-child) .md-subhead {
    color: '{{foreground-2}}'; }
