/*!
 * Angular Material Design
 * https://github.com/angular/material
 * @license MIT
 * v1.0.1
 */
a.md-button.md-THEME_NAME-theme:not([disabled]):hover,
.md-button.md-THEME_NAME-theme:not([disabled]):hover {
  background-color: '{{background-500-0.2}}'; }

a.md-button.md-THEME_NAME-theme:not([disabled]).md-focused,
.md-button.md-THEME_NAME-theme:not([disabled]).md-focused {
  background-color: '{{background-500-0.2}}'; }

a.md-button.md-THEME_NAME-theme:not([disabled]).md-icon-button:hover,
.md-button.md-THEME_NAME-theme:not([disabled]).md-icon-button:hover {
  background-color: transparent; }

a.md-button.md-THEME_NAME-theme.md-fab,
.md-button.md-THEME_NAME-theme.md-fab {
  background-color: '{{accent-color}}';
  color: '{{accent-contrast}}'; }
  a.md-button.md-THEME_NAME-theme.md-fab md-icon,
  .md-button.md-THEME_NAME-theme.md-fab md-icon {
    color: '{{accent-contrast}}'; }
  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover,
  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover {
    background-color: '{{accent-color}}'; }
  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]).md-focused,
  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]).md-focused {
    background-color: '{{accent-A700}}'; }

a.md-button.md-THEME_NAME-theme.md-primary,
.md-button.md-THEME_NAME-theme.md-primary {
  color: '{{primary-color}}'; }
  a.md-button.md-THEME_NAME-theme.md-primary.md-raised, a.md-button.md-THEME_NAME-theme.md-primary.md-fab,
  .md-button.md-THEME_NAME-theme.md-primary.md-raised,
  .md-button.md-THEME_NAME-theme.md-primary.md-fab {
    color: '{{primary-contrast}}';
    background-color: '{{primary-color}}'; }
    a.md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]) md-icon, a.md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]) md-icon,
    .md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]) md-icon,
    .md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]) md-icon {
      color: '{{primary-contrast}}'; }
    a.md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]):hover, a.md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]):hover,
    .md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]):hover,
    .md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]):hover {
      background-color: '{{primary-color}}'; }
    a.md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]).md-focused, a.md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]).md-focused,
    .md-button.md-THEME_NAME-theme.md-primary.md-raised:not([disabled]).md-focused,
    .md-button.md-THEME_NAME-theme.md-primary.md-fab:not([disabled]).md-focused {
      background-color: '{{primary-600}}'; }
  a.md-button.md-THEME_NAME-theme.md-primary:not([disabled]) md-icon,
  .md-button.md-THEME_NAME-theme.md-primary:not([disabled]) md-icon {
    color: '{{primary-color}}'; }

a.md-button.md-THEME_NAME-theme.md-fab,
.md-button.md-THEME_NAME-theme.md-fab {
  background-color: '{{accent-color}}';
  color: '{{accent-contrast}}'; }
  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]) .md-icon,
  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]) .md-icon {
    color: '{{accent-contrast}}'; }
  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover,
  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]):hover {
    background-color: '{{accent-color}}'; }
  a.md-button.md-THEME_NAME-theme.md-fab:not([disabled]).md-focused,
  .md-button.md-THEME_NAME-theme.md-fab:not([disabled]).md-focused {
    background-color: '{{accent-A700}}'; }

a.md-button.md-THEME_NAME-theme.md-raised,
.md-button.md-THEME_NAME-theme.md-raised {
  color: '{{background-900}}';
  background-color: '{{background-50}}'; }
  a.md-button.md-THEME_NAME-theme.md-raised:not([disabled]) .md-icon,
  .md-button.md-THEME_NAME-theme.md-raised:not([disabled]) .md-icon {
    color: '{{background-contrast}}'; }
  a.md-button.md-THEME_NAME-theme.md-raised:not([disabled]):hover,
  .md-button.md-THEME_NAME-theme.md-raised:not([disabled]):hover {
    background-color: '{{background-50}}'; }
  a.md-button.md-THEME_NAME-theme.md-raised:not([disabled]).md-focused,
  .md-button.md-THEME_NAME-theme.md-raised:not([disabled]).md-focused {
    background-color: '{{background-200}}'; }

a.md-button.md-THEME_NAME-theme.md-warn,
.md-button.md-THEME_NAME-theme.md-warn {
  color: '{{warn-color}}'; }
  a.md-button.md-THEME_NAME-theme.md-warn.md-raised, a.md-button.md-THEME_NAME-theme.md-warn.md-fab,
  .md-button.md-THEME_NAME-theme.md-warn.md-raised,
  .md-button.md-THEME_NAME-theme.md-warn.md-fab {
    color: '{{warn-contrast}}';
    background-color: '{{warn-color}}'; }
    a.md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]) md-icon, a.md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]) md-icon,
    .md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]) md-icon,
    .md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]) md-icon {
      color: '{{warn-contrast}}'; }
    a.md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]):hover, a.md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]):hover,
    .md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]):hover,
    .md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]):hover {
      background-color: '{{warn-color}}'; }
    a.md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]).md-focused, a.md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]).md-focused,
    .md-button.md-THEME_NAME-theme.md-warn.md-raised:not([disabled]).md-focused,
    .md-button.md-THEME_NAME-theme.md-warn.md-fab:not([disabled]).md-focused {
      background-color: '{{warn-700}}'; }
  a.md-button.md-THEME_NAME-theme.md-warn:not([disabled]) md-icon,
  .md-button.md-THEME_NAME-theme.md-warn:not([disabled]) md-icon {
    color: '{{warn-color}}'; }

a.md-button.md-THEME_NAME-theme.md-accent,
.md-button.md-THEME_NAME-theme.md-accent {
  color: '{{accent-color}}'; }
  a.md-button.md-THEME_NAME-theme.md-accent.md-raised, a.md-button.md-THEME_NAME-theme.md-accent.md-fab,
  .md-button.md-THEME_NAME-theme.md-accent.md-raised,
  .md-button.md-THEME_NAME-theme.md-accent.md-fab {
    color: '{{accent-contrast}}';
    background-color: '{{accent-color}}'; }
    a.md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]) md-icon, a.md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]) md-icon,
    .md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]) md-icon,
    .md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]) md-icon {
      color: '{{accent-contrast}}'; }
    a.md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]):hover, a.md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]):hover,
    .md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]):hover,
    .md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]):hover {
      background-color: '{{accent-color}}'; }
    a.md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]).md-focused, a.md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]).md-focused,
    .md-button.md-THEME_NAME-theme.md-accent.md-raised:not([disabled]).md-focused,
    .md-button.md-THEME_NAME-theme.md-accent.md-fab:not([disabled]).md-focused {
      background-color: '{{accent-700}}'; }
  a.md-button.md-THEME_NAME-theme.md-accent:not([disabled]) md-icon,
  .md-button.md-THEME_NAME-theme.md-accent:not([disabled]) md-icon {
    color: '{{accent-color}}'; }

a.md-button.md-THEME_NAME-theme[disabled], a.md-button.md-THEME_NAME-theme.md-raised[disabled], a.md-button.md-THEME_NAME-theme.md-fab[disabled], a.md-button.md-THEME_NAME-theme.md-accent[disabled], a.md-button.md-THEME_NAME-theme.md-warn[disabled],
.md-button.md-THEME_NAME-theme[disabled],
.md-button.md-THEME_NAME-theme.md-raised[disabled],
.md-button.md-THEME_NAME-theme.md-fab[disabled],
.md-button.md-THEME_NAME-theme.md-accent[disabled],
.md-button.md-THEME_NAME-theme.md-warn[disabled] {
  color: '{{foreground-3}}' !important;
  cursor: default; }
  a.md-button.md-THEME_NAME-theme[disabled] md-icon, a.md-button.md-THEME_NAME-theme.md-raised[disabled] md-icon, a.md-button.md-THEME_NAME-theme.md-fab[disabled] md-icon, a.md-button.md-THEME_NAME-theme.md-accent[disabled] md-icon, a.md-button.md-THEME_NAME-theme.md-warn[disabled] md-icon,
  .md-button.md-THEME_NAME-theme[disabled] md-icon,
  .md-button.md-THEME_NAME-theme.md-raised[disabled] md-icon,
  .md-button.md-THEME_NAME-theme.md-fab[disabled] md-icon,
  .md-button.md-THEME_NAME-theme.md-accent[disabled] md-icon,
  .md-button.md-THEME_NAME-theme.md-warn[disabled] md-icon {
    color: '{{foreground-3}}'; }

a.md-button.md-THEME_NAME-theme.md-raised[disabled], a.md-button.md-THEME_NAME-theme.md-fab[disabled],
.md-button.md-THEME_NAME-theme.md-raised[disabled],
.md-button.md-THEME_NAME-theme.md-fab[disabled] {
  background-color: '{{foreground-4}}'; }

a.md-button.md-THEME_NAME-theme[disabled],
.md-button.md-THEME_NAME-theme[disabled] {
  background-color: transparent; }
